// A fan game by poret. This game is not affiliated with SNK. This is a fan game which is not being sold or distributed for profit.

#pragma once

#include "CoreMinimal.h"
#include "KOFTeam.h"
#include "Character/KOFBaseCharacter.h"
#include "GameFramework/GameModeBase.h"
#include "KOFDefaultGameMode.generated.h"

/**
 * 
 */
UCLASS()
class KOFFOREVER_API AKOFDefaultGameMode : public AGameModeBase
{
	GENERATED_BODY()

public:
	AKOFDefaultGameMode();
	
	TArray<AKOFBaseCharacter*>& GetP1Team() { return P1Team; }

	TArray<AKOFBaseCharacter*>& GetP2Team() { return P2Team; }

	// this will need to change when we actually have real time character switching
	AKOFBaseCharacter* GetP1CurrentCharacter() { return P1Team.Num() > 0 ? P1Team[0] : nullptr; }

	// this will need to change when we actually have real time character switching
	AKOFBaseCharacter* GetP2CurrentCharacter() { return P2Team.Num() > 0 ? P2Team[0] : nullptr; }

	AKOFBaseCharacter* GetCurrentOpponent(bool bIsPlayer1) { return bIsPlayer1 ? GetP2CurrentCharacter() : GetP1CurrentCharacter(); }

	FORCEINLINE float GetMaxAllowedDistanceFromOpponent() { return MaxAllowedDistanceFromOpponent; }

	FORCEINLINE float GetStageLength() { return StageLength; }
	
protected:
	/** Number of characters on a single player's team */
	UPROPERTY(EditAnywhere, BlueprintReadOnly, Category="Settings|Team")
	int32 NumCharactersPerTeam;

	/** Max distance the currently controlled player characters can be from one another */
	UPROPERTY(EditDefaultsOnly, BlueprintReadOnly, Category="Settings|Stage")
	float MaxAllowedDistanceFromOpponent;

	UPROPERTY(EditDefaultsOnly, BlueprintReadOnly, Category="Settings|Stage")
	float StageLength;
	
	/** Y offset from origin where character's will be spawned */
	UPROPERTY(EditDefaultsOnly, BlueprintReadOnly, Category="Settings|Stage")
	float StartY;

	UPROPERTY(EditAnywhere, BlueprintReadWrite, Category="Settings|Team")
	TArray<TSubclassOf<AKOFBaseCharacter>> P1TeamTemplate;

	/** List of references to the player's team members */
	UPROPERTY(BlueprintReadOnly)
	TArray<AKOFBaseCharacter*> P1Team;
	
	UPROPERTY(EditAnywhere, BlueprintReadWrite, Category="Settings|Team")
	TArray<TSubclassOf<AKOFBaseCharacter>> P2TeamTemplate;

	/** List of references to the player's team members */
	UPROPERTY(BlueprintReadOnly)
	TArray<AKOFBaseCharacter*> P2Team;
	
	void InitTeams();
	
	virtual void StartPlay() override;	
};
